@@ -110,7 +110,6 @@ async fn main() {
 
     let config = NodeConfig {
         name: Some("Jotaro".into()),
-        listener_ip: "127.0.0.1".parse().unwrap(),
         max_handshake_time_ms: 10_000,
         ..Default::default()
     };
@@ -118,7 +117,6 @@ async fn main() {
 
     let config = NodeConfig {
         name: Some("Dio".into()),
-        listener_ip: "127.0.0.1".parse().unwrap(),
         max_handshake_time_ms: 10_000,
         ..Default::default()
     };
@@ -132,7 +130,7 @@ async fn main() {
 
     jotaro
         .node()
-        .connect(dio.node().listening_addr())
+        .connect(dio.node().listening_addr().unwrap())
         .await
         .unwrap();
 
@@ -141,7 +139,7 @@ async fn main() {
     jotaro
         .node()
         .send_direct_message(
-            dio.node().listening_addr(),
+            dio.node().listening_addr().unwrap(),
             Bytes::copy_from_slice(&[BattleCry::Ora as u8]),
         )
         .unwrap();
