@@ -14,7 +14,7 @@ use tracing_subscriber::filter::LevelFilter;
 use pea2pea::{
     connect_nodes,
     protocols::{Handshaking, Reading, Writing},
-    Connection, ConnectionSide, Node, NodeConfig, Pea2Pea, Topology,
+    Connection, ConnectionSide, Node, Pea2Pea, Topology,
 };
 
 use std::{
@@ -49,13 +49,8 @@ struct Player {
 
 impl Player {
     async fn new() -> Self {
-        let config = NodeConfig {
-            listener_ip: "127.0.0.1".parse().unwrap(),
-            ..Default::default()
-        };
-
         Self {
-            node: Node::new(Some(config)).await.unwrap(),
+            node: Node::new(None).await.unwrap(),
             other_players: Default::default(),
             potato_count: Default::default(),
         }
@@ -64,7 +59,7 @@ impl Player {
     fn throw_potato(&self) {
         let message = Message::IHaveThePotato(self.node().name().into());
         let message = bincode::serialize(&message).unwrap();
-        self.node().send_broadcast(message.into()).unwrap();
+        self.node().send_broadcast(message.into());
 
         let (new_carrier_name, new_carrier_addr) = self
             .other_players
