@@ -73,7 +73,6 @@ impl SecureNode {
     async fn new(name: &str) -> io::Result<Self> {
         let config = NodeConfig {
             name: Some(name.into()),
-            listener_ip: "127.0.0.1".parse().unwrap(),
             conn_read_buffer_size: NOISE_BUF_LEN + 2, // 2 for the encrypted message length,
             ..Default::default()
         };
@@ -236,7 +235,7 @@ async fn main() {
     // connect the initiator to the responder
     initiator
         .node()
-        .connect(responder.node().listening_addr())
+        .connect(responder.node().listening_addr().unwrap())
         .await
         .unwrap();
 
@@ -247,7 +246,7 @@ async fn main() {
     let msg = b"why hello there, fellow noise protocol user; I'm the initiator";
     initiator
         .node()
-        .send_direct_message(responder.node().listening_addr(), msg[..].into())
+        .send_direct_message(responder.node().listening_addr().unwrap(), msg[..].into())
         .unwrap();
 
     // send a message from responder to initiator; determine the latter's address first
