@@ -8,7 +8,7 @@ use tracing_subscriber::filter::LevelFilter;
 use pea2pea::{
     connect_nodes,
     protocols::{Reading, Writing},
-    Node, NodeConfig, Pea2Pea, Topology,
+    Node, Pea2Pea, Topology,
 };
 
 use std::{convert::TryInto, io, net::SocketAddr, time::Duration};
@@ -85,12 +85,8 @@ async fn main() {
     common::start_logger(LevelFilter::INFO);
 
     let mut players = Vec::with_capacity(NUM_PLAYERS);
-    let config = NodeConfig {
-        listener_ip: "127.0.0.1".parse().unwrap(),
-        ..Default::default()
-    };
     for _ in 0..NUM_PLAYERS {
-        let player = Player(Node::new(Some(config.clone())).await.unwrap());
+        let player = Player(Node::new(None).await.unwrap());
         players.push(player);
     }
 
@@ -107,7 +103,7 @@ async fn main() {
     players[0]
         .node()
         .send_direct_message(
-            players[1].node().listening_addr(),
+            players[1].node().listening_addr().unwrap(),
             message.as_bytes().into(),
         )
         .unwrap();
