@@ -1,7 +1,7 @@
 use crate::{
     connections::{Connection, ConnectionSide, Connections},
-    protocols::{ProtocolHandler, Protocols},
-    KnownPeers, NodeConfig, NodeStats,
+    protocols::{ProtocolHandler, Protocols, ReturnableConnection},
+    KnownPeers, NodeConfig, Stats,
 };
 
 use bytes::Bytes;
@@ -25,7 +25,7 @@ use std::{
 };
 
 macro_rules! enable_protocol {
-    ($protocol_name: expr, $handler_type: ident, $node:expr, $conn: expr) => {
+    ($handler_type: ident, $node:expr, $conn: expr) => {
         if let Some(handler) = $node.protocols.$handler_type.get() {
             let (conn_returner, conn_retriever) = oneshot::channel();
 
@@ -64,7 +64,7 @@ pub struct InnerNode {
     /// The node's configuration.
     config: NodeConfig,
     /// The node's listening address.
-    listening_addr: SocketAddr,
+    listening_addr: Option<SocketAddr>,
     /// Contains objects used by the protocols implemented by the node.
     protocols: Protocols,
     /// A list of connections that have not been finalized yet.
@@ -74,7 +74,7 @@ pub struct InnerNode {
     /// Collects statistics related to the node's peers.
     known_peers: KnownPeers,
     /// Collects statistics related to the node itself.
-    stats: NodeStats,
+    stats: Stats,
     /// The node's tasks.
     pub(crate) tasks: Mutex<Vec<JoinHandle<()>>>,
 }
@@ -93,30 +93,41 @@ impl Node {
         let span = create_span(config.name.as_deref().unwrap());
 
         // procure a listening address
-        let listener_ip = config.listener_ip;
-        let listener = if let Some(port) = config.desired_listening_port {
-            let desired_listening_addr = SocketAddr::new(listener_ip, port);
-            match TcpListener::bind(desired_listening_addr).await {
-                Ok(listener) => listener,
-                Err(e) => {
-                    if config.allow_random_port {
-                        warn!(parent: span.clone(), "trying any port, the desired one is unavailable: {}", e);
-                        let random_available_addr = SocketAddr::new(listener_ip, 0);
-                        TcpListener::bind(random_available_addr).await?
-                    } else {
-                        error!(parent: span.clone(), "the desired port is unavailable: {}", e);
-                        return Err(e);
+        let listener = if let Some(listener_ip) = config.listener_ip {
+            let listener = if let Some(port) = config.desired_listening_port {
+                let desired_listening_addr = SocketAddr::new(listener_ip, port);
+                match TcpListener::bind(desired_listening_addr).await {
+                    Ok(listener) => listener,
+                    Err(e) => {
+                        if config.allow_random_port {
+                            warn!(parent: span.clone(), "trying any port, the desired one is unavailable: {}", e);
+                            let random_available_addr = SocketAddr::new(listener_ip, 0);
+                            TcpListener::bind(random_available_addr).await?
+                        } else {
+                            error!(parent: span.clone(), "the desired port is unavailable: {}", e);
+                            return Err(e);
+                        }
                     }
                 }
-            }
-        } else if config.allow_random_port {
-            let random_available_addr = SocketAddr::new(listener_ip, 0);
-            TcpListener::bind(random_available_addr).await?
+            } else if config.allow_random_port {
+                let random_available_addr = SocketAddr::new(listener_ip, 0);
+                TcpListener::bind(random_available_addr).await?
+            } else {
+                panic!(
+                    "you must either provide a desired port or allow a random port to be chosen"
+                );
+            };
+
+            Some(listener)
         } else {
-            panic!("you must either provide a desired port or allow a random port to be chosen");
+            None
         };
 
-        let listening_addr = listener.local_addr()?;
+        let listening_addr = if let Some(ref listener) = listener {
+            Some(listener.local_addr()?)
+        } else {
+            None
+        };
 
         let node = Node(Arc::new(InnerNode {
             span,
@@ -130,42 +141,48 @@ impl Node {
             tasks: Default::default(),
         }));
 
-        let node_clone = node.clone();
-        let listening_task = tokio::spawn(async move {
-            trace!(parent: node_clone.span(), "spawned the listening task");
-            loop {
-                match listener.accept().await {
-                    Ok((stream, addr)) => {
-                        debug!(parent: node_clone.span(), "tentatively accepted a connection from {}", addr);
+        if let Some(listener) = listener {
+            let node_clone = node.clone();
+            let listening_task = tokio::spawn(async move {
+                trace!(parent: node_clone.span(), "spawned the listening task");
+                loop {
+                    match listener.accept().await {
+                        Ok((stream, addr)) => {
+                            debug!(parent: node_clone.span(), "tentatively accepted a connection from {}", addr);
 
-                        if !node_clone.can_add_connection() {
-                            debug!(parent: node_clone.span(), "rejecting the connection from {}", addr);
-                            continue;
-                        }
-
-                        node_clone.connecting.lock().insert(addr);
-
-                        let node_clone2 = node_clone.clone();
-                        task::spawn(async move {
-                            if let Err(e) = node_clone2
-                                .adapt_stream(stream, addr, ConnectionSide::Responder)
-                                .await
-                            {
-                                node_clone2.connecting.lock().remove(&addr);
-                                node_clone2.known_peers().register_failure(addr);
-                                error!(parent: node_clone2.span(), "couldn't accept a connection: {}", e);
+                            if !node_clone.can_add_connection() {
+                                debug!(parent: node_clone.span(), "rejecting the connection from {}", addr);
+                                continue;
                             }
-                        });
-                    }
-                    Err(e) => {
-                        error!(parent: node_clone.span(), "couldn't accept a connection: {}", e);
+
+                            node_clone.connecting.lock().insert(addr);
+
+                            let node_clone2 = node_clone.clone();
+                            task::spawn(async move {
+                                if let Err(e) = node_clone2
+                                    .adapt_stream(stream, addr, ConnectionSide::Responder)
+                                    .await
+                                {
+                                    node_clone2.connecting.lock().remove(&addr);
+                                    node_clone2.known_peers().register_failure(addr);
+                                    error!(parent: node_clone2.span(), "couldn't accept a connection: {}", e);
+                                }
+                            });
+                        }
+                        Err(e) => {
+                            error!(parent: node_clone.span(), "couldn't accept a connection: {}", e);
+                        }
                     }
                 }
-            }
-        });
-        node.tasks.lock().push(listening_task);
+            });
+            node.tasks.lock().push(listening_task);
 
-        debug!(parent: node.span(), "the node is ready; listening on {}", listening_addr);
+            debug!(parent: node.span(), "the node is ready");
+        }
+
+        if let Some(listening_addr) = node.listening_addr {
+            debug!(parent: node.span(), "listening on port {}", listening_addr);
+        }
 
         Ok(node)
     }
@@ -182,7 +199,7 @@ impl Node {
     }
 
     /// Returns a reference to the node's stats.
-    pub fn stats(&self) -> &NodeStats {
+    pub fn stats(&self) -> &Stats {
         &self.stats
     }
 
@@ -191,15 +208,17 @@ impl Node {
         &self.span
     }
 
-    /// Returns the node's listening address.
-    pub fn listening_addr(&self) -> SocketAddr {
+    /// Returns the node's listening address; returns an error if the node was configured
+    /// to not listen for inbound connections.
+    pub fn listening_addr(&self) -> io::Result<SocketAddr> {
         self.listening_addr
+            .ok_or_else(|| io::ErrorKind::AddrNotAvailable.into())
     }
 
     async fn enable_protocols(&self, conn: Connection) -> io::Result<Connection> {
-        let conn = enable_protocol!("HandshakeProtocol", handshake_handler, self, conn);
-        let conn = enable_protocol!("ReadingProtocol", reading_handler, self, conn);
-        let conn = enable_protocol!("WritingProtocol", writing_handler, self, conn);
+        let conn = enable_protocol!(handshake_handler, self, conn);
+        let conn = enable_protocol!(reading_handler, self, conn);
+        let conn = enable_protocol!(writing_handler, self, conn);
 
         Ok(conn)
     }
@@ -237,23 +256,24 @@ impl Node {
 
         self.connections.add(connection);
         self.connecting.lock().remove(&peer_addr);
-        self.known_peers.register_connection(peer_addr);
 
         Ok(())
     }
 
     /// Connects to the provided `SocketAddr`.
     pub async fn connect(&self, addr: SocketAddr) -> io::Result<()> {
-        if addr == self.listening_addr()
-            || addr.ip().is_loopback() && addr.port() == self.listening_addr().port()
-        {
-            error!(parent: self.span(), "can't connect to node's own listening address ({})", addr);
-            return Err(io::ErrorKind::AddrInUse.into());
+        if let Ok(listening_addr) = self.listening_addr() {
+            if addr == listening_addr
+                || addr.ip().is_loopback() && addr.port() == listening_addr.port()
+            {
+                error!(parent: self.span(), "can't connect to node's own listening address ({})", addr);
+                return Err(io::ErrorKind::AddrInUse.into());
+            }
         }
 
         if !self.can_add_connection() {
             error!(parent: self.span(), "too many connections; refusing to connect to {}", addr);
-            return Err(io::ErrorKind::Other.into());
+            return Err(io::ErrorKind::PermissionDenied.into());
         }
 
         if self.connections.is_connected(addr) {
@@ -285,11 +305,20 @@ impl Node {
     }
 
     /// Disconnects from the provided `SocketAddr`.
-    pub fn disconnect(&self, addr: SocketAddr) -> bool {
+    pub async fn disconnect(&self, addr: SocketAddr) -> bool {
+        if let Some(handler) = self.protocols.disconnect_handler.get() {
+            if self.is_connected(addr) {
+                let (sender, receiver) = oneshot::channel();
+
+                handler.send((addr, sender)).await;
+                let _ = receiver.await; // can't really fail
+            }
+        }
+
         let disconnected = self.connections.remove(addr);
 
         if disconnected {
-            info!(parent: self.span(), "disconnected from {}", addr);
+            debug!(parent: self.span(), "disconnected from {}", addr);
         } else {
             warn!(parent: self.span(), "wasn't connected to {}", addr);
         }
@@ -302,22 +331,21 @@ impl Node {
         self.connections
             .sender(addr)?
             .try_send(message)
-            .map_err(|_| {
-                error!(parent: self.span(), "the outbound channel of {} is full; message dropped.", addr);
+            .map_err(|e| {
+                error!(parent: self.span(), "can't send a message to {}: {}", addr, e);
+                self.stats().register_failure();
                 io::ErrorKind::Other.into()
             })
     }
 
     /// Broadcasts the provided message to all peers, as long as the `Writing` protocol is enabled.
-    pub fn send_broadcast(&self, message: Bytes) -> io::Result<()> {
+    pub fn send_broadcast(&self, message: Bytes) {
         for (message_sender, addr) in self.connections.senders() {
-            // an error means the connection is shutting down, which is already reported in logs
-            let _ = message_sender.try_send(message.clone()).map_err(|_| {
-                error!(parent: self.span(), "the outbound channel of {} is full; message dropped.", addr);
+            let _ = message_sender.try_send(message.clone()).map_err(|e| {
+                error!(parent: self.span(), "can't send a message to {}: {}", addr, e);
+                self.stats().register_failure();
             });
         }
-
-        Ok(())
     }
 
     /// Returns a list containing addresses of active connections.
@@ -358,28 +386,38 @@ impl Node {
     }
 
     /// Sets up the handshake handler, as part of the `Handshaking` protocol.
-    pub fn set_handshake_handler(&self, handler: ProtocolHandler) {
+    pub fn set_handshake_handler(&self, handler: ProtocolHandler<ReturnableConnection>) {
         if self.protocols.handshake_handler.set(handler).is_err() {
             panic!("the handshake_handler field was set more than once!");
         }
     }
 
     /// Sets up the reading handler, as part of enabling the `Reading` protocol.
-    pub fn set_reading_handler(&self, handler: ProtocolHandler) {
+    pub fn set_reading_handler(&self, handler: ProtocolHandler<ReturnableConnection>) {
         if self.protocols.reading_handler.set(handler).is_err() {
             panic!("the reading_handler field was set more than once!");
         }
     }
 
     /// Sets up the writing handler, as part of enabling the `Writing` protocol.
-    pub fn set_writing_handler(&self, handler: ProtocolHandler) {
+    pub fn set_writing_handler(&self, handler: ProtocolHandler<ReturnableConnection>) {
         if self.protocols.writing_handler.set(handler).is_err() {
             panic!("the writing_handler field was set more than once!");
         }
     }
 
+    /// Sets up the disconnect handler, as part of enabling the `Disconnect` protocol.
+    pub fn set_disconnect_handler(
+        &self,
+        handler: ProtocolHandler<(SocketAddr, oneshot::Sender<()>)>,
+    ) {
+        if self.protocols.disconnect_handler.set(handler).is_err() {
+            panic!("the disconnect_handler field was set more than once!");
+        }
+    }
+
     /// Gracefully shuts the node down.
-    pub fn shut_down(&self) {
+    pub async fn shut_down(&self) {
         debug!(parent: self.span(), "shutting down");
 
         let mut tasks = std::mem::take(&mut *self.tasks.lock()).into_iter();
@@ -388,7 +426,7 @@ impl Node {
         }
 
         for addr in self.connected_addrs() {
-            self.disconnect(addr);
+            self.disconnect(addr).await;
         }
 
         for handle in tasks {
