@@ -49,15 +49,10 @@ where
                     let node = processing_clone.node();
                     trace!(parent: node.span(), "spawned a task for processing messages from {}", addr);
 
-                    loop {
-                        if let Some(msg) = inbound_message_receiver.recv().await {
-                            if let Err(e) = processing_clone.process_message(addr, msg).await {
-                                error!(parent: node.span(), "can't process an inbound message: {}", e);
-                                node.known_peers().register_failure(addr);
-                            }
-                        } else {
-                            node.disconnect(addr);
-                            break;
+                    while let Some(msg) = inbound_message_receiver.recv().await {
+                        if let Err(e) = processing_clone.process_message(addr, msg).await {
+                            error!(parent: node.span(), "can't process a message from {}: {}", addr, e);
+                            node.known_peers().register_failure(addr);
                         }
                     }
                 });
@@ -93,7 +88,7 @@ where
                             Err(e) => {
                                 node.known_peers().register_failure(addr);
                                 if node.config().fatal_io_errors.contains(&e.kind()) {
-                                    node.disconnect(addr);
+                                    node.disconnect(addr).await;
                                     break;
                                 } else {
                                     sleep(Duration::from_secs(
@@ -162,9 +157,9 @@ where
                             self.node().stats().register_received_message(len);
 
                             // send the message for further processing
-                            if message_sender.send(msg).await.is_err() {
-                                error!(parent: self.node().span(), "the inbound message channel is closed");
-                                return Err(io::ErrorKind::BrokenPipe.into());
+                            if let Err(e) = message_sender.try_send(msg) {
+                                error!(parent: self.node().span(), "can't process a message from {}: {}", addr, e);
+                                self.node().stats().register_failure();
                             }
 
                             // if the read is exhausted, reset the carry and return
