@@ -19,23 +19,6 @@ static RANDOM_BYTES: Lazy<Bytes> = Lazy::new(|| {
     )
 });
 
-#[derive(Clone)]
-struct Spammer(Node);
-
-impl Pea2Pea for Spammer {
-    fn node(&self) -> &Node {
-        &self.0
-    }
-}
-
-impl Writing for Spammer {
-    fn write_message(&self, _: SocketAddr, payload: &[u8], buffer: &mut [u8]) -> io::Result<usize> {
-        buffer[..4].copy_from_slice(&(payload.len() as u32).to_le_bytes());
-        buffer[4..][..payload.len()].copy_from_slice(payload);
-        Ok(4 + payload.len())
-    }
-}
-
 #[derive(Clone)]
 struct Sink(Node);
 
@@ -88,12 +71,14 @@ async fn run_bench_scenario(sender_count: usize) -> f64 {
 
     let config = NodeConfig {
         conn_write_buffer_size: MSG_SIZE,
-        listener_ip: "127.0.0.1".parse().unwrap(),
         conn_outbound_queue_depth: NUM_MESSAGES,
         ..Default::default()
     };
     let spammers = common::start_nodes(sender_count, Some(config)).await;
-    let spammers = spammers.into_iter().map(Spammer).collect::<Vec<_>>();
+    let spammers = spammers
+        .into_iter()
+        .map(common::MessagingNode)
+        .collect::<Vec<_>>();
 
     for spammer in &spammers {
         spammer.enable_writing();
@@ -102,7 +87,6 @@ async fn run_bench_scenario(sender_count: usize) -> f64 {
     let config = NodeConfig {
         conn_read_buffer_size: MSG_SIZE * 3,
         max_connections: sender_count as u16,
-        listener_ip: "127.0.0.1".parse().unwrap(),
         ..Default::default()
     };
     let sink = Sink(Node::new(Some(config)).await.unwrap());
@@ -112,14 +96,14 @@ async fn run_bench_scenario(sender_count: usize) -> f64 {
     for spammer in &spammers {
         spammer
             .node()
-            .connect(sink.node().listening_addr())
+            .connect(sink.node().listening_addr().unwrap())
             .await
             .unwrap();
     }
 
     wait_until!(10, sink.node().num_connected() == sender_count);
 
-    let sink_addr = sink.node().listening_addr();
+    let sink_addr = sink.node().listening_addr().unwrap();
 
     let start = Instant::now();
     for spammer in spammers {
