@@ -1,5 +1,6 @@
 use bytes::Bytes;
 use parking_lot::Mutex;
+use tokio::time::sleep;
 use tracing::*;
 
 mod common;
@@ -9,7 +10,7 @@ use pea2pea::{
 };
 use TestMessage::*;
 
-use std::{collections::HashSet, io, net::SocketAddr, sync::Arc};
+use std::{collections::HashSet, io, net::SocketAddr, sync::Arc, time::Duration};
 
 #[derive(PartialEq, Eq, Hash, Debug, Clone, Copy)]
 enum TestMessage {
@@ -48,9 +49,9 @@ impl Reading for EchoNode {
         _source: SocketAddr,
         buffer: &[u8],
     ) -> io::Result<Option<(Self::Message, usize)>> {
-        let bytes = common::read_len_prefixed_message(2, buffer)?;
+        let bytes = common::read_len_prefixed_message(4, buffer)?;
 
-        Ok(bytes.map(|bytes| (TestMessage::from(bytes[2]), bytes.len())))
+        Ok(bytes.map(|bytes| (TestMessage::from(bytes[4]), bytes.len())))
     }
 
     async fn process_message(&self, source: SocketAddr, message: Self::Message) -> io::Result<()> {
@@ -72,15 +73,15 @@ impl Reading for EchoNode {
 
 impl Writing for EchoNode {
     fn write_message(&self, _: SocketAddr, payload: &[u8], buffer: &mut [u8]) -> io::Result<usize> {
-        buffer[..2].copy_from_slice(&(payload.len() as u16).to_le_bytes());
-        buffer[2..][..payload.len()].copy_from_slice(payload);
-        Ok(2 + payload.len())
+        buffer[..4].copy_from_slice(&(payload.len() as u32).to_le_bytes());
+        buffer[4..][..payload.len()].copy_from_slice(payload);
+        Ok(4 + payload.len())
     }
 }
 
 #[tokio::test]
 async fn messaging_example() {
-    tracing_subscriber::fmt::init();
+    // tracing_subscriber::fmt::init();
 
     let shouter = common::MessagingNode::new("shout").await;
     shouter.enable_reading();
@@ -88,7 +89,6 @@ async fn messaging_example() {
 
     let picky_echo_config = NodeConfig {
         name: Some("picky_echo".into()),
-        listener_ip: "127.0.0.1".parse().unwrap(),
         ..Default::default()
     };
     let picky_echo = EchoNode {
@@ -98,7 +98,7 @@ async fn messaging_example() {
     picky_echo.enable_reading();
     picky_echo.enable_writing();
 
-    let picky_echo_addr = picky_echo.node().listening_addr();
+    let picky_echo_addr = picky_echo.node().listening_addr().unwrap();
 
     shouter.node().connect(picky_echo_addr).await.unwrap();
 
@@ -127,24 +127,22 @@ async fn messaging_example() {
 #[tokio::test]
 async fn drop_connection_on_invalid_message() {
     let reader = common::MessagingNode::new("reader").await;
+    let reader_addr = reader.node().listening_addr().unwrap();
     reader.enable_reading();
+
     let writer = common::MessagingNode::new("writer").await;
     writer.enable_writing();
 
-    writer
-        .node()
-        .connect(reader.node().listening_addr())
-        .await
-        .unwrap();
+    writer.node().connect(reader_addr).await.unwrap();
 
     wait_until!(1, reader.node().num_connected() == 1);
 
     // an invalid message: a zero-length payload
-    let bad_message: &'static [u8] = &[];
+    let bad_message = Bytes::from(vec![]);
 
     writer
         .node()
-        .send_direct_message(reader.node().listening_addr(), bad_message.into())
+        .send_direct_message(reader_addr, bad_message)
         .unwrap();
 
     wait_until!(1, reader.node().num_connected() == 0);
@@ -160,17 +158,13 @@ async fn drop_connection_on_oversized_message() {
     let config = NodeConfig {
         name: Some("reader".into()),
         conn_read_buffer_size: MSG_SIZE_LIMIT,
-        listener_ip: "127.0.0.1".parse().unwrap(),
         ..Default::default()
     };
     let reader = common::MessagingNode(Node::new(Some(config)).await.unwrap());
+    let reader_addr = reader.node().listening_addr().unwrap();
     reader.enable_reading();
 
-    writer
-        .node()
-        .connect(reader.node().listening_addr())
-        .await
-        .unwrap();
+    writer.node().connect(reader_addr).await.unwrap();
 
     wait_until!(1, reader.node().num_connected() == 1);
 
@@ -179,10 +173,7 @@ async fn drop_connection_on_oversized_message() {
 
     writer
         .node()
-        .send_direct_message(
-            reader.node().listening_addr(),
-            common::prefix_with_len(2, &oversized_payload),
-        )
+        .send_direct_message(reader_addr, common::prefix_with_len(2, &oversized_payload))
         .unwrap();
 
     wait_until!(1, reader.node().num_connected() == 0);
@@ -195,15 +186,63 @@ async fn drop_connection_on_zero_read() {
     let peer = common::MessagingNode::new("peer").await;
 
     peer.node()
-        .connect(reader.node().listening_addr())
+        .connect(reader.node().listening_addr().unwrap())
         .await
         .unwrap();
 
     wait_until!(1, reader.node().num_connected() == 1);
 
     // the peer shuts down, i.e. disconnects
-    peer.node().shut_down();
+    peer.node().shut_down().await;
 
     // the reader should drop its connection too now
     wait_until!(1, reader.node().num_connected() == 0);
 }
+
+#[tokio::test]
+async fn no_reading_no_delivery() {
+    let reader = common::MessagingNode::new("defunct reader").await;
+    let reader_addr = reader.node().listening_addr().unwrap();
+
+    let writer = common::MessagingNode::new("writer").await;
+    writer.enable_writing();
+
+    writer.node().connect(reader_addr).await.unwrap();
+
+    wait_until!(1, reader.node().num_connected() == 1);
+
+    // writer sends a message
+    writer
+        .node()
+        .send_direct_message(reader_addr, common::prefix_with_len(4, &[0; 16]))
+        .unwrap();
+
+    sleep(Duration::from_millis(10)).await;
+
+    // but the reader didn't enable reading, so it won't receive anything
+    wait_until!(1, reader.node().stats().received() == (0, 0));
+}
+
+#[tokio::test]
+async fn no_writing_no_delivery() {
+    let reader = common::MessagingNode::new("reader").await;
+    let reader_addr = reader.node().listening_addr().unwrap();
+    reader.enable_reading();
+
+    let writer = common::MessagingNode::new("defunct writer").await;
+
+    writer.node().connect(reader_addr).await.unwrap();
+
+    wait_until!(1, reader.node().num_connected() == 1);
+
+    // writer tries to send a message
+    assert!(writer
+        .node()
+        .send_direct_message(reader_addr, common::prefix_with_len(4, &[0; 16]))
+        .is_err());
+
+    sleep(Duration::from_millis(10)).await;
+
+    // the writer didn't enable writing, so the reader won't receive anything
+    wait_until!(1, reader.node().stats().received() == (0, 0));
+}
